@@ -1,8 +1,29 @@
+/*
+ *  Copyright (C) 2019 ~ 2021 Uniontech Software Technology Co.,Ltd
+ *
+ * Author:
+ *
+ * Maintainer:
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 package main
 
 import (
-	"bufio"
 	"bytes"
+	"encoding/json"
 	"errors"
 	"flag"
 	"fmt"
@@ -10,6 +31,7 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"regexp"
 	"runtime"
 	"strconv"
 	"strings"
@@ -18,14 +40,14 @@ import (
 	"./bootloader/grubcfg"
 	"./bootloader/pmoncfg"
 	"github.com/godbus/dbus"
+	"github.com/linuxdeepin/dde-api/inhibit_hint"
 	login1 "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.login1"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/log"
+	"github.com/linuxdeepin/go-lib/procfs"
+	"github.com/linuxdeepin/go-lib/strv"
+	"github.com/linuxdeepin/go-lib/utils"
 	"golang.org/x/xerrors"
-	"pkg.deepin.io/dde/api/inhibit_hint"
-	"pkg.deepin.io/lib/dbusutil"
-	"pkg.deepin.io/lib/log"
-	"pkg.deepin.io/lib/procfs"
-	"pkg.deepin.io/lib/strv"
-	"pkg.deepin.io/lib/utils"
 )
 
 var logger = log.NewLogger("ab-recovery")
@@ -34,6 +56,7 @@ const (
 	configFile              = "/etc/deepin/ab-recovery.json"
 	backupMountPoint        = "/deepin-ab-recovery-backup"
 	abRecoveryGrubCfgFile   = "/etc/default/grub.d/11_deepin_ab_recovery.cfg"
+	abRecoveryGrubCfg12File = "/etc/default/grub.d/12_deepin_ab_recovery.cfg"
 	abRecoveryFile          = "/usr/lib/deepin-daemon/ab-recovery"
 	ddeWelcomeFile          = "/usr/lib/deepin-daemon/dde-welcome"
 	abKernelBackupDir       = "/boot/kernel-backup/"
@@ -62,15 +85,32 @@ var options struct {
 	printShHideOs  bool
 }
 
-var _extraDirs = []struct {
-	originDir       string
-	hospiceChildDir string
-}{
+type extraDir struct {
+	originDir       string   // 需要备份的文件夹
+	hospiceChildDir string   // 为空时,和base(originDir)一致
+	specifiedFiles  []string // 该切片内只能存放originDir中的文件或文件夹名
+}
+
+var _extraDirs = []extraDir{
 	{
 		originDir: "/var/lib/systemd",
 	},
+	{
+		originDir: "/var/uos",
+		specifiedFiles: []string{
+			"os-license",
+		},
+	},
 }
 
+const backupRecordPath = "/var/lib/deepin-ab-recovery/record.json"
+
+var _lastBackUpRecord map[string]string
+var _currentBackUpRecord map[string]string
+
+var _renameFailedMsgRegexp = regexp.MustCompile(`rsync: rename "([0-9a-zA-Z/+.=-]+)" -> "([0-9a-zA-Z/+.=-]+)": Operation not permitted`)
+var _delFailedMsgRegexp = regexp.MustCompile(`rsync: delete_file: unlink[(]([0-9a-zA-Z/+.=-]+)[)] failed: Operation not permitted`)
+
 func init() {
 	flag.BoolVar(&options.noRsync, "no-rsync", false, "")
 	flag.BoolVar(&options.noGrubMkconfig, "no-grub-mkconfig", false, "")
@@ -111,6 +151,31 @@ func printShHideOs() (exitCode int) {
 		fmt.Printf("GRUB_OS_PROBER_SKIP_LIST=\"$GRUB_OS_PROBER_SKIP_LIST %s@%s\"\n", uuid, device)
 		return
 	}
+	// 没有找到备份分区的情况,默认将rootb分区作为备份分区
+	uuid, err := getUuidByLabel("rootb")
+	if err != nil {
+		logWarningf("get rootb uuid error: %v", err)
+		exitCode = 3
+		return
+	}
+	mountPoint, err := getMountPointByLabel("rootb")
+	if err != nil {
+		logWarningf("get rootb mountPoint error: %v", err)
+		exitCode = 4
+		return
+	}
+	if strings.TrimSpace(mountPoint) == "/" {
+		logWarningf("Cannot use rootb as a backup partition")
+		exitCode = 5
+		return
+	}
+	device, err := getDeviceByUuid(uuid)
+	if err != nil {
+		logWarningf("get backup device by backup uuid error: %v", err)
+		exitCode = 6
+		return
+	}
+	fmt.Printf("GRUB_OS_PROBER_SKIP_LIST=\"$GRUB_OS_PROBER_SKIP_LIST %s@%s\"\n", uuid, device)
 	return
 }
 
@@ -160,12 +225,12 @@ func mountDevice(device, dir string) (fn func(), err error) {
 func umountDeleteDir(dir string) {
 	err := exec.Command("umount", dir).Run()
 	if err != nil {
-		logWarningf("failed to unmount directory %q: %v", dir, err)
+		logWarningf("failed to umount directory %q: %v", dir, err)
 	}
 
 	err = os.Remove(dir)
 	if err != nil {
-		logWarningf("failed to remove backup mount point: %v", err)
+		logWarningf("failed to remove unmounted directory: %v", err)
 	}
 }
 
@@ -301,7 +366,7 @@ func backup(cfg *Config, envVars []string) error {
 	}()
 
 	skipDirs := []string{
-		"/media", "/tmp",
+		"/media", "/tmp", "/proc", "/sys", "/dev", "/run",
 	}
 
 	tmpExcludeFile, err := writeExcludeFile(append(skipDirs, backupMountPoint))
@@ -342,10 +407,40 @@ func backup(cfg *Config, envVars []string) error {
 		return xerrors.Errorf("failed to save config file %q: %w", configFile, err)
 	}
 
-	backupAllExtraDirs()
-
-	err = runRsync(tmpExcludeFile)
+	initBackUpRecord(backupRecordPath, defaultHospiceDir)
+	recoverDeprecatedFilesOrDirs(backupRecordPath, false)
+	err = updateBackUpRecordFile(backupRecordPath)
 	if err != nil {
+		logger.Warning(err)
+		return err
+	}
+	backupExtra()
+	errMsg, err := runRsync(tmpExcludeFile)
+	if err != nil {
+		allMatchedString := _renameFailedMsgRegexp.FindAllStringSubmatch(errMsg, -1)
+		for _, matchString := range allMatchedString {
+			if len(matchString) == 3 {
+				tempFilePath := matchString[1]
+				destFilePath := matchString[2]
+				if strings.Contains(filepath.Base(tempFilePath), filepath.Base(destFilePath)) {
+					err := exec.Command("chattr", "-i", filepath.Join(backupMountPoint, destFilePath)).Run()
+					if err != nil {
+						logger.Warning(err)
+						continue
+					}
+				}
+			}
+		}
+		allMatchedString = _delFailedMsgRegexp.FindAllStringSubmatch(errMsg, -1)
+		for _, matchString := range allMatchedString {
+			if len(matchString) == 2 {
+				err := exec.Command("chattr", "-i", filepath.Join(backupMountPoint, matchString[1])).Run()
+				if err != nil {
+					logger.Warning(err)
+					continue
+				}
+			}
+		}
 		return xerrors.Errorf("run rsync err: %w", err)
 	}
 
@@ -365,20 +460,6 @@ func backup(cfg *Config, envVars []string) error {
 	if err != nil {
 		return xerrors.Errorf("failed to modify fs tab: %w", err)
 	}
-	var rulesPaths = []string{
-		"/etc/udev/rules.d/80-udisks2.rules",
-		"/etc/udev/rules.d/80-udisks-installer.rules",
-	}
-	for _, rulesPath := range rulesPaths {
-		_, err := os.Stat(rulesPath)
-		if err == nil {
-			err = modifyRules(filepath.Join(backupMountPoint, rulesPath), cfg.Current)
-			if err != nil {
-				return xerrors.Errorf("failed to modify rules: %w", err)
-			}
-			break
-		}
-	}
 
 	kFiles, err := backupKernel()
 	if err != nil {
@@ -399,101 +480,40 @@ func backup(cfg *Config, envVars []string) error {
 	return nil
 }
 
-// 备份所有不在根分区的额外文件夹
-func backupAllExtraDirs() {
-	for _, item := range _extraDirs {
-		err := backupExtraDir(item.originDir, item.hospiceChildDir, defaultHospiceDir)
-		if err != nil {
-			logWarningf("backup extra dir %q failed: %v", item.originDir, err)
-		}
-	}
-}
-
 // 备份不在根分区的额外文件夹，比如实际上在 /data 分区的 /var/lib/systemd 文件夹。
-func backupExtraDir(originDir, hospiceChildDir, hospiceDir string) error {
-	if hospiceChildDir == "" {
-		hospiceChildDir = filepath.Base(originDir)
-		if hospiceChildDir == "" {
-			return errors.New("hospiceChildDir is empty")
-		}
-	}
-	isSym, err := isSymlink(originDir)
-	if err != nil {
-		return xerrors.Errorf("isSymlink %q failed: %w", originDir, err)
-	}
-	if isSym {
-		return nil
-	}
-	err = os.MkdirAll(hospiceDir, 0755)
-	if err != nil {
-		return xerrors.Errorf("make hospice dir failed: %w", err)
-	}
-	hDir := filepath.Join(hospiceDir, hospiceChildDir)
-	// 先删除一遍, 确保 hDir 不存在。
-	err = os.RemoveAll(hDir)
-	if err != nil {
-		return xerrors.Errorf("remove dir failed: %w", err)
-	}
-
-	err = exec.Command("cp", "-a", originDir, hDir).Run()
-	if err != nil {
-		return xerrors.Errorf("run cp command failed: %w", err)
-	}
-	return nil
-}
-
-// 系统备份升级后还原，由于备份分区未被隐藏，导致文件管理器显示的系统盘不是真正的系统盘
-// 显示的是备份分区的系统盘,所以在备份时,修改备份分区下的rule文件,将备份分区继续继续隐藏
-func modifyRules(filename, Uuid string) error {
-	in, err := os.Open(filename)
-	if err != nil {
-		logger.Warning(err)
-		return err
-	}
-	defer in.Close()
-
-	out, err := os.OpenFile(filename+".new", os.O_RDWR|os.O_CREATE, 0644)
-	if err != nil {
-		logger.Warning(err)
-		return err
-	}
-	defer out.Close()
-
-	var (
-		index   int
-		scanner = bufio.NewScanner(in)
-	)
-
-	var newLine string
-	for scanner.Scan() {
-		line := scanner.Text()
-
-		if index == 0 && string(line) == "# hide rootb" {
-			newLine = "# hide ab-recovery backup partition"
-		} else if index == 1 {
-			newLine = fmt.Sprintf("ENV{ID_FS_UUID}==%q, ENV{UDISKS_IGNORE}=\"1\"", Uuid)
-		} else {
-			newLine = string(line)
-		}
-		_, err = out.WriteString(newLine + "\n")
+func backupExtra() {
+	for origin, backupPath := range _currentBackUpRecord {
+		isSym, err := isSymlink(origin)
 		if err != nil {
-			logger.Warning(err)
-			return err
+			logger.Warningf("isSymlink %q failed: %v", origin, err)
+			continue
+		}
+		if isSym {
+			continue
+		}
+		err = os.MkdirAll(filepath.Dir(backupPath), 0755)
+		if err != nil {
+			logger.Warningf("make backup dir failed: %v", err)
+			continue
+		}
+		err = os.RemoveAll(backupPath)
+		if err != nil {
+			logger.Warningf("remove dir failed: %v", err)
+			continue
+		}
+		err = exec.Command("cp", "-a", origin, backupPath).Run()
+		if err != nil {
+			logger.Warningf("run cp command failed: %v", err)
+			continue
 		}
-		index++
 	}
-	err = os.Rename(filename+".new", filename)
-	if err != nil {
-		logger.Warning(err)
-		return err
-	}
-	return nil
 }
 
-func runRsync(excludeFile string) error {
+func runRsync(excludeFile string) (string, error) {
+	var errBuffer bytes.Buffer
 	if options.noRsync {
 		logger.Debug("skip run rsync")
-		return nil
+		return "", nil
 	}
 
 	logger.Debug("run rsync...")
@@ -501,18 +521,16 @@ func runRsync(excludeFile string) error {
 	if logger.GetLogLevel() == log.LevelDebug {
 		rsyncArgs = append(rsyncArgs, "-v")
 	}
-	rsyncArgs = append(rsyncArgs, "-x", "-a", "--delete-after",
+	rsyncArgs = append(rsyncArgs, "-X", "-x", "-a", "--delete-after",
 		"--exclude-from="+excludeFile,
 		"/", backupMountPoint+"/")
 
 	cmd := exec.Command("rsync", rsyncArgs...)
 	cmd.Stdout = os.Stdout
-	cmd.Stderr = os.Stderr
+	cmd.Stderr = &errBuffer
+	cmd.Env = append(cmd.Env, "LC_ALL=C")
 	err := cmd.Run()
-	if err != nil {
-		return err
-	}
-	return nil
+	return errBuffer.String(), err
 }
 
 func backupKernel() (kFiles *kernelFiles, err error) {
@@ -757,6 +775,10 @@ func fixBackup() error {
 	if err != nil {
 		return err
 	}
+	err = utils.CopyFile(abRecoveryGrubCfg12File, filepath.Join(backupMountPoint, abRecoveryGrubCfg12File))
+	if err != nil {
+		return err
+	}
 	// 暂时屏蔽真dde-welcome运行
 	backupDDEWelcomeFile := filepath.Join(backupMountPoint, ddeWelcomeFile)
 	backupDDEWelcomeFileInfo, err := os.Stat(backupDDEWelcomeFile)
@@ -841,15 +863,9 @@ func restore(cfg *Config, envVars []string) error {
 	if err != nil {
 		return xerrors.Errorf("failed to write grub cfg: %w", err)
 	}
-
-	restoreAllExtraDirs()
-
-	// delete cache archive files
-	err = exec.Command("/usr/bin/lastore-apt-clean", "-force-delete").Run()
-	if err != nil {
-		logger.Warning("failed to delete archive files:", err)
-	}
-
+	initBackUpRecord(backupRecordPath, defaultHospiceDir)
+	recoverDeprecatedFilesOrDirs(backupRecordPath, true)
+	restoreExtra()
 	// swap current and backup
 	cfg.Current, cfg.Backup = cfg.Backup, cfg.Current
 	cfg.Time = nil
@@ -859,61 +875,97 @@ func restore(cfg *Config, envVars []string) error {
 		return xerrors.Errorf("failed to save config file %q: %w", configFile, err)
 	}
 
+	// 还原时，对需要隐藏的分区进行处理: 将备份分区进行隐藏，并解除挂载
+	var rulesPaths = []string{
+		"/etc/udev/rules.d/80-udisks2.rules",
+		"/etc/udev/rules.d/80-udisks-installer.rules",
+	}
+	foundRules := false
+
+	rootDisk, err := getPathDisk("/")
+	if err != nil {
+		return xerrors.Errorf("failed to get root disk: %w", err)
+	}
+
+	labelUuidMap, err := getLabelUuidMap(rootDisk)
+	if err != nil {
+		return xerrors.Errorf("failed to get label uuid map: %w", err)
+	}
+	backupDevice, err := getDeviceByUuid(cfg.Backup)
+	if err != nil {
+		logger.Warning(err)
+		return err
+	}
+	backupLabel, err := getDeviceLabel(backupDevice)
+	if err != nil {
+		logger.Warning(err)
+		return err
+	}
+	for _, rulesPath := range rulesPaths {
+		_, err = os.Stat(rulesPath)
+		if err == nil {
+			err = modifyRules(rulesPath, labelUuidMap, cfg.Backup, cfg.Current, backupLabel)
+			if err != nil {
+				return xerrors.Errorf("failed to modify rules: %w", err)
+			}
+			foundRules = true
+		}
+	}
+	if !foundRules {
+		logger.Warning("not found 80-udisks-installer.rules or 80-udisks2.rules")
+	} else {
+		err = reloadUdev() // 重载udev的rules,让rules的修改生效
+		if err != nil {
+			logger.Warning(err)
+			return err
+		}
+		mountDir, err := getMountPointByLabel(strings.ToLower(strings.TrimSpace(backupLabel)))
+		if err != nil {
+			logger.Warning(err)
+		} else {
+			umountDeleteDir(mountDir)
+		}
+	}
+	// end
+
 	err = os.Remove(filepath.Join("/", backupPartitionMarkFile))
 	if err != nil && !os.IsNotExist(err) {
 		return xerrors.Errorf("failed to delete backup partition mark file: %w", err)
 	}
 
-	return nil
-}
+	adapterActivator()
 
-func restoreAllExtraDirs() {
-	for _, item := range _extraDirs {
-		err := restoreExtraDir(item.originDir, item.hospiceChildDir, defaultHospiceDir)
-		if err != nil {
-			logWarningf("restore extra dir %q failed: %v", item.originDir, err)
-		}
-	}
+	return nil
 }
 
 // 回退不在根分区的额外文件夹，实际上是通过创建软链接完成的。
 // 如果已经是软链接了，则不需要处理。
-func restoreExtraDir(originDir, hospiceChildDir, hospiceDir string) error {
-	if hospiceChildDir == "" {
-		hospiceChildDir = filepath.Base(originDir)
-		if hospiceChildDir == "" {
-			return errors.New("hospiceChildDir is empty")
+func restoreExtra() {
+	for origin, backupPath := range _lastBackUpRecord {
+		isSym, err := isSymlink(origin)
+		if err != nil {
+			logger.Warningf("isSymlink %q failed: %v", origin, err)
+			continue
+		}
+		if isSym {
+			continue
+		}
+		_, err = os.Stat(backupPath)
+		if err != nil {
+			logger.Warningf("stat backup path failed: %v", err)
+			continue
+		}
+		err = os.RemoveAll(origin)
+		if err != nil {
+			logger.Warningf("remove origin dir failed: %v", err)
+			continue
+		}
+		err = os.Symlink(backupPath, origin)
+		if err != nil {
+			logger.Warningf("create symlink for %q failed: %v", origin, err)
+			continue
 		}
 	}
-	isSym, err := isSymlink(originDir)
-	if err != nil {
-		return xerrors.Errorf("isSymlink %q failed: %w", originDir, err)
-	}
-
-	if isSym {
-		return nil
-	}
-
-	// 检查 hDir 文件夹是否存在，它必须存在。
-	hDir := filepath.Join(hospiceDir, hospiceChildDir)
-	hDirInfo, err := os.Stat(hDir)
-	if err != nil {
-		return xerrors.Errorf("stat hDir failed: %w", err)
-	}
-	if !hDirInfo.IsDir() {
-		return errors.New("hDir is not a directory")
-	}
-
-	err = os.RemoveAll(originDir)
-	if err != nil {
-		return xerrors.Errorf("remove origin dir failed: %w", err)
-	}
-
-	err = os.Symlink(hDir, originDir)
-	if err != nil {
-		return xerrors.Errorf("create symlink for %q failed: %w", originDir, err)
-	}
-	return nil
 }
 
 func writeBootloaderCfgRestore(currentUuid, currentDevice, backupUuid string, envVars []string) error {
@@ -1225,3 +1277,141 @@ func getRollbackMenuTextForceEn(osDesc string, backupTime time.Time) string {
 	dateTimeStr := backupTime.Format("Mon 02 Jan 2006 03:04:05 PM MST")
 	return fmt.Sprintf(msgRollBack, osDesc, dateTimeStr)
 }
+
+// 根据备份记录,还原修改
+func recoverDeprecatedFilesOrDirs(recordPath string, isRestore bool) {
+	const oldBackupPath = "/usr/share/deepin-ab-recovery/hospice/uos"
+	if !isExist(recordPath) { // 如果不存在该文件,则为兼容旧版本时使用
+		// 兼容 /var/uos文件夹备份改为 /var/uos/os-license文件备份
+		// 处理软链接和非软链接两种情况
+		isSym, err := isSymlink("/var/uos")
+		if err != nil {
+			logger.Warningf("isSymlink %q failed: %v", "/var/uos", err)
+			return
+		}
+		if isSym {
+			err := os.RemoveAll("/var/uos")
+			if err != nil {
+				logger.Warningf("remove origin dir failed: %v", err)
+				return
+			}
+			err = exec.Command("mv", oldBackupPath, "/var").Run()
+			if err != nil {
+				logger.Warningf("mv backup dir to origin dir failed: %v", err)
+				return
+			}
+		} else {
+			if isRestore {
+				err := exec.Command("mv", filepath.Join(oldBackupPath, "os-license"), "/var/uos", "-f").Run() // 将os-license文件还原至备份时候的状态
+				if err != nil {
+					logger.Warningf("only restore os-license failed: %v", err)
+					return
+				}
+			}
+			err := os.RemoveAll(oldBackupPath)
+			if err != nil {
+				logger.Warningf("remove origin dir failed: %v", err)
+				return
+			}
+		}
+	}
+
+	for originPath, backupPath := range _lastBackUpRecord {
+		// 判断新旧版本备份内容是否存在差异
+		if currentBackupPath, ok := _currentBackUpRecord[originPath]; ok && backupPath == currentBackupPath {
+			continue
+		}
+		// 恢复之前的备份
+		isSym, err := isSymlink(originPath)
+		if err != nil {
+			logger.Warningf("isSymlink %q failed: %v", originPath, err)
+			continue
+		}
+		if isSym {
+			err = os.RemoveAll(originPath)
+			if err != nil {
+				logger.Warningf("remove origin dir failed: %v", err)
+				continue
+			}
+			err = exec.Command("cp", "-a", backupPath, originPath).Run()
+			if err != nil {
+				logger.Warningf("run cp command failed: %v", err)
+				continue
+			}
+		}
+		err = os.RemoveAll(backupPath)
+		if err != nil {
+			logger.Warningf("remove backup file or dir failed: %v", err)
+			continue
+		}
+	}
+	return
+}
+
+// 更新记录备份项的文件
+func updateBackUpRecordFile(path string) error {
+	err := os.MkdirAll(filepath.Dir(path), 0755)
+	if err != nil {
+		return err
+	}
+	data, err := json.Marshal(_currentBackUpRecord)
+	if err != nil {
+		return err
+	}
+	return ioutil.WriteFile(path, data, 0644)
+}
+
+// 初始化_lastBackUpRecord和_currentBackUpRecord
+func initBackUpRecord(recordPath, hospice string) {
+	// 最新备份配置
+	_currentBackUpRecord = make(map[string]string)
+	for _, item := range _extraDirs {
+		if item.specifiedFiles != nil {
+			var hospiceChildDir string
+			if item.hospiceChildDir == "" {
+				hospiceChildDir = filepath.Base(item.originDir)
+			} else {
+				hospiceChildDir = item.hospiceChildDir
+			}
+			for _, file := range item.specifiedFiles {
+				_currentBackUpRecord[filepath.Join(item.originDir, file)] = filepath.Join(hospice, hospiceChildDir, file)
+			}
+		} else {
+			var hospiceChildDir string
+			if item.hospiceChildDir == "" {
+				hospiceChildDir = filepath.Base(item.originDir)
+			} else {
+				hospiceChildDir = item.hospiceChildDir
+			}
+			_currentBackUpRecord[item.originDir] = filepath.Join(hospice, hospiceChildDir)
+		}
+	}
+	// 备份记录
+	_lastBackUpRecord = make(map[string]string)
+	content, err := ioutil.ReadFile(recordPath)
+	if err != nil {
+		if os.IsNotExist(err) {
+			logger.Info(err)
+		} else {
+			logger.Warningf("read %s file failed: %v", recordPath, err)
+		}
+		return
+	}
+	err = json.Unmarshal(content, &_lastBackUpRecord)
+	if err != nil {
+		logger.Warningf("unmarshal %s file to json failed: %v", recordPath, err)
+		return
+	}
+}
+
+// 在还原过程中适配系统激活
+func adapterActivator() {
+	err := exec.Command("pkill", "-ef", "/usr/lib/deepin-daemon/uos-license-agent").Run()
+	if err != nil {
+		logger.Warning("failed to kill uos-license-agent:", err)
+	}
+	err = exec.Command("pkill", "-ef", "/usr/bin/uos-activator").Run()
+	if err != nil {
+		logger.Warning("failed to kill uos-activator:", err)
+	}
+}
